<?php
namespace FiloBlu\Storelocator\Controller\Adminhtml\Stores;

use Magento\Backend\App\Action\Context;
use Magento\Ui\Component\MassAction\Filter;
use FiloBlu\Storelocator\Model\ResourceModel\Stores\CollectionFactory;
use Magento\Framework\Controller\ResultFactory;
use FiloBlu\Storelocator\Controller\Adminhtml\Stores\Geocode;
use FiloBlu\Storelocator\Helper\Data;

/**
 * Class MassGeocode
 */
class MassGeocode extends \Magento\Backend\App\Action
{
   /**
     * @var Filter
     */
    protected $filter;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

     /**
     * @var context
     */
    protected $context;

     /**
     * @var geocode
     */
    protected $geocode;

     /**
     * @var geocode
     */
    protected $helper;


    /**
     * @param Context $context
     * @param Filter $filter
     * @param CollectionFactory $collectionFactory
     */

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('FiloBlu_Storelocator::storelocator_save');
    }

    /**
     * Delete action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */

    public function __construct(Context $context, Geocode $geocode, Data $helper, Filter $filter, CollectionFactory $collectionFactory)
    {
        $this->geocode = $geocode;
        $this->helper = $helper;
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        parent::__construct($context);
    }
    /**
     * Execute action
     *
     * @return \Magento\Backend\Model\View\Result\Redirect
     * @throws \Magento\Framework\Exception\LocalizedException|\Exception
     */

     
    public function execute()
    {   
        
        $collection = $this->filter->getCollection($this->collectionFactory->create());
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
       
        $msgSuccess = "";
        $msgNotFound = "";

        foreach ($collection as $item) 
        {   
            
            $id = $item->getData('store_id');

            if ($id) {
                
                 try {
                     $model = $this->_objectManager->create('FiloBlu\Storelocator\Model\Stores');
                     $model->load($id);
                     $address = $model->getData("street") . " " .
                         $model->getData("postcode") . " " .
                         $model->getData("city") . " " .
                         $model->getData("country");
                     
                     $geocodeData = $this->helper->geocode($address);
                    
                     if ($geocodeData) {
                    
                         $model->setLatitude($geocodeData[0]);
                         $model->setLongitude($geocodeData[1]);
                         $model->save();
                         $msgSuccess .= "</br>" . "Geolocation data has been updated. Address found ".$id . " - " . $geocodeData[2];                        
                         
                     }
                     else {
                         $msgNotFound .= "</br>" . "We can\'t find an address to update geolocation data for id " . $id;
                     }                    
                    
                 } catch (\Exception $e) {
                     $this->messageManager->addError($e->getMessage());
                     return $resultRedirect->setPath('*/*/');
                 }
             }
        }

        if ($msgSuccess) {
            $this->messageManager->addSuccess($msgSuccess);
         }

         if ($msgNotFound) {
            $this->messageManager->addError($msgNotFound);
         }
        
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        return $resultRedirect->setPath('*/*/');
    }
}